/* global Components, Services, APP_STARTUP, APP_SHUTDOWN */
/* exported console, startup, shutdown, install, uninstall */

"use strict";

var app, DOM_observer, Cu = Components.utils, Ci = Components.interfaces;

Cu["import"]("resource://gre/modules/Services.jsm");

try {
	// this isn't available under version ~16
	Cu["import"]("resource://gre/modules/devtools/Console.jsm");

	// from version 24, there is Browser Console
	if (parseFloat(Services.appinfo.platformVersion) < 24) {
		throw new Error;
	}
}
catch (ex) {
	this.console = {
		_toStr: function(s) {
			if (typeof s === "object" && s !== null) {
				s = JSON.stringify(s);
			}

			return s;
		},
		log: function(m, s) {
			Services.console.logStringMessage((m && s ? m : "") + this._toStr(s || m));
		},
		info: function(s) {
			this.log(":", s);
		},
		warn: function(s) {
			this.log("!", s);
		},
		error: function(s) {
			Cu.reportError(this._toStr(s));
		}
	};
}

function startup(data, reason) {
	var initBG = function(ev) {
		if (ev) {
			this.removeEventListener("load", initBG, false);
		}

		app = Object.freeze({
			platform: Services.appinfo.name,
			platform_version: parseFloat(Services.appinfo.platformVersion),
			baseURI: "chrome://viewhance/content/",
			name: "Viewhance"
		});

		if (app.platform_version < 10) {
			Components.manager.addBootstrappedManifestLocation(data.installPath);
		}

		var systemPrincipal = Components.classes["@mozilla.org/systemprincipal;1"].createInstance(Ci.nsIPrincipal);
		var hiddenWindow = (Services.appShell
			|| Components.classes["@mozilla.org/appshell/appShellService;1"].getService(Ci.nsIAppShellService)).hiddenDOMWindow;
		var bgProcess = hiddenWindow.document.createElementNS("http://www.w3.org/1999/xhtml", "iframe");

		(bgProcess.wrappedJSObject || bgProcess).initBgProcess = function(bg_win) {
			this.initBgProcess = null;

			bg_win.console = console;
			bg_win.extension = Object.freeze({
				app: app,
				_: function (bundle) {
					var stringBundle = Services.strings
							.createBundle("chrome://" + app.name + "/locale/" + bundle + ".properties");

					return function(s) {
						try {
							return stringBundle.GetStringFromName(s);
						} catch (ex) {
							return s;
						}
					};
				}
			});

			var contentContext = Object.freeze({
				app: app,
				postMessage: function(e) {
					try {
						bg_win.Port.postMessage(e);
					} catch(ex) {
						// console.error(app.name + ": (port error) " + ex);
					}
				}
			});

			DOM_observer = {
				observe: function(win/*, topic*/) {
					if (!(win.document instanceof win.ImageDocument
							|| win.document instanceof win.HTMLDocument
								&& (win.location.protocol === "http:"
									|| win.location.protocol === "https:"
									|| win.location.protocol === "file:"
									|| win.location.protocol === "ftp:"))) {
						return;
					}

					win = Cu.Sandbox(systemPrincipal, {
						sandboxPrototype: win,
						wantComponents: false,
						wantXHRConstructor: false
					});

					win.extension = contentContext;

					// for older Firefox
					if (!win.console) {
						win.console = console;
					}

					var lss = Services.scriptloader.loadSubScript;

					lss(app.baseURI + "includes/app.js", win);

					if (win.document instanceof win.ImageDocument
						|| win.document.querySelector("body>video:only-child:not([src])")) {
						lss(app.baseURI + "includes/viewer.js", win);
					}
					else {
						lss(app.baseURI + "includes/opener.js", win);
					}
				}
			};

			Services.obs.addObserver(DOM_observer, "content-document-global-created", false);

			// On install, inject content scripts into already opened tabs
			/*if (reason === ADDON_INSTALL) {
				let enmrtr = Services.wm.getEnumerator("navigator:browser");

				while (enmrtr.hasMoreElements()) {
					let browsers = enmrtr.getNext().getBrowser().browsers, i = browsers.length;

					while (i--) {
						let win = browsers[i].contentWindow;

						if (win.ImageDocument && win.document instanceof win.ImageDocument) {
							DOM_observer.observe(win);
						}
					}
				}
			}*/
		};

		bgProcess.setAttribute("id", app.name + "_bgProcess");
		bgProcess.setAttribute("src", app.baseURI + "background.html");
		hiddenWindow.document.documentElement.appendChild(bgProcess);
	};

	if (reason === APP_STARTUP) {
		var observer = {
			observe: function(subject, topic) {
				if (topic === "domwindowopened") {
					Services.ww.unregisterNotification(observer);
					subject.QueryInterface(Ci.nsIDOMWindow).addEventListener("load", initBG, false);
				}
			}
		};

		Services.ww.registerNotification(observer);
	}
	else {
		initBG();
	}
}

function shutdown(data, reason) {
	if (reason === APP_SHUTDOWN) {
		return;
	}

	Services.strings.flushBundles();

	var enmrtr = Services.wm.getEnumerator("navigator:browser");

	// close extension tabs
	while (enmrtr.hasMoreElements()) {
		var gBrowser = enmrtr.getNext().gBrowser, tabs = gBrowser.tabs, i = tabs.length;

		while (i--) {
			if (gBrowser.getBrowserForTab(tabs[i]).contentWindow.location.host === app.name.toLowerCase()) {
				gBrowser.removeTab(tabs[i]);
			}
		}
	}

	if (app.platform_version < 10) {
		Components.manager.removeBootstrappedManifestLocation(data.installPath);
	}

	if (DOM_observer) {
		Services.obs.removeObserver(DOM_observer, "content-document-global-created");
	}

	var bgFrame = (Services.appShell || Components.classes["@mozilla.org/appshell/appShellService;1"].getService(Ci.nsIAppShellService)).hiddenDOMWindow.document.getElementById(app.name + "_bgProcess");

	if (bgFrame) {
		bgFrame.parentNode.removeChild(bgFrame);
	}

	DOM_observer = bgFrame = null;
}

function install() {
	// https://bugzilla.mozilla.org/show_bug.cgi?id=719376
	Services.strings.flushBundles();
}

function uninstall() {}
